/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.mediatypes;

import org.wso2.registry.RegistryException;
import org.wso2.registry.Resource;
import org.wso2.registry.jdbc.dao.VersionedResourceDAO;
import org.wso2.registry.utils.AuthorizationUtil;
import org.wso2.usermanager.Realm;

import javax.sql.DataSource;

/**
 * Base class of all media type handlers. It defines the three basic methods to be implemented by
 * all media type handlers. This is initialized with the objects required for handling media type
 * specific operations, so that extending media type handlers can easily access them. One such
 * object is the instance of the MediaTypeManager.
 */
public abstract class MediaTypeHandler {

    /**
     * Data source for obtaining database connections. Media type handlers need to access the
     * database for performing almost all the tasks.
     */
    protected DataSource dataSource;

    /**
     * User manager realm, which can be used to set permissions on resources. Media type handlers
     * may want to add other resources (e.g. imports) than the originally given resource. So it is
     * necessary to allow media type handler to set permissions for the resources it is dealing
     * with.
     */
    protected Realm realm;

    /**
     * While processing media types, handlers may want to access resources with other media types.
     * Then they can just call the appropriate methods of the MediaTypeManager for doing that.
     * <p/>
     * e.g. "text" mediatype handler may just want to change the content of the resource from String
     * to byte[] before PUT. So that users can set Strings as the content of text resources. But
     * after changing that, text handler want PUT the resource using normal process. In that case,
     * text handler can execute the put(...) method of the DefaultMediaTypeHandler after doing the
     * relevent changes to the resource.
     */
    protected MediaTypeManager mediaTypeManager;

    /** ResourceDAO for directly accessing resources. */
    //protected ResourceDAO resourceDAO = new ResourceDAO();
    protected VersionedResourceDAO resourceDAO = new VersionedResourceDAO();

    /** Some common authorization tasks are implemented in this util. */
    protected AuthorizationUtil authorizationUtil = new AuthorizationUtil();

    public MediaTypeHandler(DataSource dataSource,
                            Realm realm,
                            MediaTypeManager mediaTypeManager) {

        this.dataSource = dataSource;
        this.realm = realm;
        this.mediaTypeManager = mediaTypeManager;
    }

    /**
     * Processes the GET action of the media type.
     *
     * @param path        Path of the resource
     * @param rawArtifact Raw artifact at the given path, which is retrieved directly from the
     *                    database. This can be used by media type handlers, which need to alter
     *                    contents of the raw artifact, etc.
     * @return Resource instance if the media type handler processes the GET action. If resource is
     *         returned it should perform all tasks to fulfill the GET request. Returns null if the
     *         media type handler does not process the GET action. If null is returned, get(...)
     *         method of the DefaultMediaTypeHandler will be invoked.
     * @throws RegistryException
     */
    public abstract Resource get(String path, Resource rawArtifact) throws RegistryException;

    /**
     * Processes the PUT action of the media type.
     *
     * @param path     to put the resource.
     * @param resource to put.
     * @return true if the media type handler processes the PUT action. If true is returned it
     *         should perform all tasks to fulfill the PUT request. Returns false if the media type
     *         handler does not process the PUT action. If false is returned, put(...) method of the
     *         DefaultMediaTypeHandler will be invoked.
     * @throws RegistryException
     */
    public abstract boolean put(String path, Resource resource) throws RegistryException;

    /**
     * Creates a resource in the given path by fetching the resource content from the given URL.
     *
     * @param path      Path to add the new resource.
     * @param sourceURL URL to fetch the resource content
     * @param metadata
     * @return true if the media type handler processes the import action. If true is returned it
     *         should perform all tasks to fulfill the import request. Returns false if the media
     *         type handler does not process the import action. If false is returned,
     *         importResource(...) method of the DefaultMediaTypeHandler will be invoked.
     * @throws RegistryException
     */
    public abstract String importResource(String path, String sourceURL, Resource metadata)
            throws RegistryException;

    /**
     * Processes the DELETE action of the media type.
     *
     * @param path path of the resource to be deleted.
     * @return true if the media type handler processes the DELETE action. If true is returned it
     *         should perform all tasks to fulfill the DELETE request. Returns false if the media
     *         type handler does not process the DELETE action. If false is returned, delete(...)
     *         method of the DefaultMediaTypeHandler will be invoked.
     * @throws RegistryException
     */
    public abstract boolean delete(String path) throws RegistryException;

    /**
     * Invokes when a child resource is added. Only the media type handlers of collection resources
     * may have a meaningfull implementation of this method.
     *
     * @param childPath Path of the child resource
     * @param resource  Child resource
     * @return True if the child resource is allowed to be added. False if the child resource is
     *         rejected.
     * @throws RegistryException
     */
    public abstract boolean putChild(String childPath, Resource resource) throws RegistryException;

    /**
     * Invokes when a child resource is imported. Only the media type handlers of collection
     * resources may have a meaningfull implementation of this method.
     *
     * @param childPath Path of the child resource
     * @param sourceURL URL to import child resource content
     * @return True if the child resource is allowed to be imported. False if the child resource is
     *         rejected.
     * @throws RegistryException
     */
    public abstract boolean importChild(String childPath, String sourceURL)
            throws RegistryException;
}
